GameGrid = class(function(self,_sizeX,_sizeY)
	self.tiles = {}
	for x=1,_sizeX do
		self.tiles[x] = {}
		for y=1,_sizeY do
			self.tiles[x][y] = false
		end
	end
	self.size = {x=_sizeX,y=_sizeY}
	self.tilesize = G_WH/21
	self.currentTile = false
	self.nextTile = GameTile(math.random(1,7),HSV(math.random(0,255),255,255))
	return self
end)

GameGrid.Render = function(self)
	love.graphics.setLineWidth( self.tilesize/10 )
		local n = self.tilesize/10
		local f = self.tilesize/2
	for x=1,self.size.x do
		for y=1,self.size.y do
			if self.tiles[x][y] then
				if not G_GameEnded then
				love.graphics.setColor(150,150,150,255)
				else
				love.graphics.setColor(200,0,0,255)
				end
			else
				if self.currentTile and self:IsCurrentTileAt(x,y) then
						love.graphics.setColor(self.currentTile.colour.r,self.currentTile.colour.g,self.currentTile.colour.b,200)
					if self.currentTile.pos.x == x and self.currentTile.pos.y == y then
						love.graphics.setColor(self.currentTile.colour.r,self.currentTile.colour.g,self.currentTile.colour.b,255)
					end
				else
					love.graphics.setColor(50,50,50,255)
				end
			end
			self:RenderTile(x,y,n,f)
		end
	end
	if self.nextTile then
		self:RenderNextTile(8,1,n,f)
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle( "line", self.tilesize-f, self.tilesize-f, self.tilesize*10, self.tilesize*20, n,n,5 )
end

GameGrid.RenderTile = function(self,x,y,n,f)
	love.graphics.rectangle( "fill", x*self.tilesize-f, y*self.tilesize-f, self.tilesize-1, self.tilesize-1, n,n,5 )
	love.graphics.setColor(255,255,255,50)
	love.graphics.rectangle( "line", x*self.tilesize-f, y*self.tilesize-f, self.tilesize-1, self.tilesize-1, n,n,5 )
end


GameGrid.RenderNextTile = function(self,x,y,n,f)
	local t = self.nextTile:GetTileShapePos()
	for k,v in pairs(t) do
		love.graphics.setColor(self.nextTile.colour.r,self.nextTile.colour.g,self.nextTile.colour.b,255)
		self:RenderTile(v.x+x,v.y+y,n,f)
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle( "line", self.tilesize*(x+4)-f-n, self.tilesize*(y+1)-f-n, self.tilesize*4+n*2, self.tilesize*2+n*2, n,n,5 )
end

GameGrid.GetCurrentTile = function(self)
	if self.currentTile == false then
		return nil
	else
		return self.currentTile
	end
end

GameGrid.UpdateGridDown = function(self)
	if self.currentTile then
		if self:GetCurrentTileCollision(0,1) then
			self:RenderTileToGrid()
				self:ForceNewTile()
		else
			self.currentTile:Move(0,1)
		end
	else
		self:ForceNewTile()
	end
end

GameGrid.GridMove = function(self,x,y)
	if self.currentTile then
		if self:GetCurrentTileCollision(x,y) then
			if y ~= 0 then
				self:RenderTileToGrid()
				self:ForceNewTile()
			end
		else
			self.currentTile:Move(x,y)
		end
	else
		self:ForceNewTile()
	end
end

GameGrid.GridRotate = function(self)
	if self.currentTile then
		if self:GetCurrentTileRotCollision(1) then
		else
			self.currentTile:Rotate(1)
		end
	else
		self:ForceNewTile()
	end
end

GameGrid.GetGridCollision = function(self,x,y)
	if self.tiles[x] == nil then
	--	print("we are out of bounds here " .. x .. "/" ..y)
		return true
	elseif y < 1 then
		return false
	elseif self.tiles[x][y] == nil then
--		print("we are out of bounds here " .. x .. "/" ..y)
		return true
	else
			return self.tiles[x][y]
	end
end

GameGrid.IsCurrentTileAt = function(self,x,y)
	if self.currentTile then
		local t = self.currentTile:GetTileShapePos()
		for k,v in pairs(t) do
			if x == v.x and y == v.y then
				return true
			end
		end
	end
	return false
end

GameGrid.GetCurrentTileRotCollision = function(self,d)
	if self.currentTile then
		local t = self.currentTile:GetTileRotPos(d)
		for k,v in pairs(t) do
			if self:GetGridCollision(v.x,v.y) then
				return true
			end
		end
	end
	return false
end

GameGrid.GetCurrentTileCollision = function(self,dx,dy)
	if self.currentTile then
		local t = self.currentTile:GetTileShapePos()
		for k,v in pairs(t) do
			if self:GetGridCollision(dx+v.x,dy+v.y) then
			--	print("we found something here " .. dx+v.x .. "/" ..dy+v.y)
				return true
			end
		end
	end
	return false
end

GameGrid.RenderTileToGrid = function(self)
		local gameEnd = false
		if self.currentTile then
			local t = self.currentTile:GetTileShapePos()
			for k,v in pairs(t) do
				self.tiles[v.x][v.y] = true
				if v.y == 1 then gameEnd = true end
			end
		end
		if gameEnd == true then
			self:GameEnd()
		end
		self:CheckLines()
end

GameGrid.GameEnd = function(self)
	self.currentTile:Destroy()
	GameEnd()
end

GameGrid.GameStart = function(self)
		self.tiles = {}
		for x=1,self.size.x do
			self.tiles[x] = {}
			for y=1,self.size.y do
				self.tiles[x][y] = false
			end
		end
	self.currentTile = false
	self.nextTile = GameTile(math.random(1,7),HSV(math.random(0,255),255,255))
end

GameGrid.ForceNewTile = function(self,t)
		if self.currentTile then
			self.currentTile:Destroy()
		end
		if not G_GameEnded then
		self.currentTile = self.nextTile
		self.nextTile = GameTile(math.random(1,7),HSV(math.random(0,255),255,255))
		else
		self.currentTile = nil
		self.nextTile:Destroy()
		self.nextTile = nil
		end
end

GameGrid.CheckLines = function(self)
	local t = {}
	for y=1,self.size.y do
		local n = 0
		for x=1,self.size.x do
			if self.tiles[x][y] then n = n +1 end
		end
		if n == self.size.x then
			table.insert(t,y)
		end
	end
	local scored = false
		for k,i in pairs(t) do
			--DDPrint("scored line: " .. i)
			Score(k)
			scored = true
			for j=0,i-1 do
				local y = i-j
				for x=1,self.size.x do
					if self.tiles[x][y-1] ~= nil then
						self.tiles[x][y] = self.tiles[x][y-1]
					else
						self.tiles[x][y] = false
					end
				end
			end
		end
		if scored then G_SoundScore:play() end
end
